--[[
    Core
]]

local addon_name, ns = ...

local Printer = ns.Printer
local displays = ns.displays
local display = ns.display
local settings = ns.settings

--------------------------------------------------------------------------------

local function forall(func)
    for _, v in pairs(displays) do
        func(v)
    end
end

local function handleEvents(self, event, arg1, arg2, ...)
    Printer.debug(4, "Handling event: %s", event)
    if event == "ADDON_LOADED" and arg1 == addon_name then
        Printer.debug(1, "Debug level: %d", Printer.debug_level)
        if CharDisplays then
            for k, v in pairs(CharDisplays) do
                ns.CharDisplays[k] = v
            end
            display.RestoreDisplays(ns.CharDisplays)
        end
        self:UnregisterEvent("ADDON_LOADED")
    elseif event == "PLAYER_LOGOUT" then
        CharDisplays = ns.CharDisplays
        -- Save display positions
        for name, opts in pairs(CharDisplays) do
            local point, rel_to, rel_point, xoff, yoff = displays[name]:GetPoint()
            rel_to = rel_to and rel_to:GetName()
            CharDisplays[name].position = { point, rel_to, rel_point, xoff, yoff }
        end
    elseif event == "PLAYER_REGEN_DISABLED" then
        forall(function(disp)
                   display.Lock(disp)
                   if #disp.queue.frames > 0 and disp.rotation then
                       disp.rotation.next_index = 1
                       local fs = disp.queue.frames
                       display.BGColourHighlight(fs[1])
                   end
               end)
    elseif event == "PLAYER_REGEN_ENABLED" then
        forall(function(disp)
                   display.Unlock(disp)
                   if #disp.queue.frames > 0 and disp.rotation then
                       local n = disp.rotation.next_index
                       local fs = disp.queue.frames
                       display.BGColourDefault(fs[n])
                   end
               end)
    end
end

local addon = CreateFrame("Frame")
addon:RegisterEvent("ADDON_LOADED")
addon:RegisterEvent("PLAYER_REGEN_DISABLED")
addon:RegisterEvent("PLAYER_REGEN_ENABLED")
addon:RegisterEvent("PLAYER_LOGOUT")
addon:SetScript("OnEvent", handleEvents)

--- ConfigMode ---
cfgmode_to_hide = {}
CONFIGMODE_CALLBACKS = CONFIGMODE_CALLBACKS or {}
CONFIGMODE_CALLBACKS[addon_name] = function(action)
    if action == "ON" then
        forall(function(disp)
            if disp:IsHidden() then
                cfgmod_to_hide[disp] = true
            end
            display.Unlock(disp)
            display.Show()
        end)
    elseif action == "OFF" then
        forall(function(disp)
            display.Lock(disp)
            if cfgmod_to_hide[disp] then
                display.Hide(disp)
            end
        end)
    end
end

--------------------------------------------------------------------------------
-- User Control
--------------------------------------------------------------------------------

-- Slash commands
local commands = LibStub:GetLibrary("OakSlash-1.0")(addon_name)
local t = {
    new = display.New,
    close = display.Close,
    show = display.Show,
    hide = display.Hide,
    autoinclude = function(disp) display.SetAutoInclude(disp, true) end,
    noautoinclude = function(disp) display.SetAutoInclude(disp, false) end,
    rename = display.Rename,
    refill = display.Refill,
    rotation = function(disp) display.SetRotation(disp, true) end,
    norotation = function(disp) display.SetRotation(disp, false) end,
    announce = function(disp) display.SetRotation(disp, "ANNOUNCE") end,
    share = display.ShareRotation,
    debug = function(l) Printer.SetDebug(l); ns.rotation.Printer.SetDebug(l);
                        ns.FramePool.Printer.SetDebug(l) end,
    grow = { up = function(disp) display.SetDirection(disp, "up") end,
             down = function(disp) display.SetDirection(disp, "down") end,
         },
    lock = display.Lock,
    unlock = display.Unlock
}
commands:NewCommand("oi", t)
--------------------------------------------------------------------------------
